#Version 3.4 loops through one LED at a time. Includes average value and standard deviation. Lists 'failed' LEDs in the output. 
#Cleaned up commenting. 
#'all' includes all LEDs in LED list.
#Reorganizes UI.

#buttons, labels, and entry boxes
#create new frame - new objects pack within that subframe(f1,f2,f3)
#new frames create new rows here
set ff [frame $f.f1]
pack $ff -fill x
button $ff.go -text "Go" -command go
label $ff.ml -text "Minimum:"
entry $ff.me -textvariable min
label $ff.ipl -text "IP Address:"
entry $ff.ipe -textvariable LWDAQ_config_BCAM(daq_ip_addr)
pack $ff.go $ff.ml $ff.me $ff.ipl $ff.ipe -side left -expand yes
set ff [frame $f.f2]
pack $ff -fill x
label $ff.dsl -text "Driver Socket:"
entry $ff.dse -textvariable LWDAQ_config_BCAM(daq_driver_socket)
label $ff.sdsl -text "Source Driver Socket:"
entry $ff.sdse -textvariable LWDAQ_config_BCAM(daq_source_driver_socket)
pack $ff.dsl $ff.dse $ff.sdsl $ff.sdse -side left -expand yes
set ff [frame $f.f3]
pack $ff -side left
label $ff.ledl -text "LED List: "
entry $ff.lede -textvariable ledlist -width 50
pack $ff.ledl -side left -expand yes
pack $ff.lede -side left -expand yes

LWDAQ_print $t "36-Way Contact Injector Power Output Test V2" purple

#basic settings in LWDAQ
set LWDAQ_config_BCAM(analysis_num_spots) {1}
set LWDAQ_info_BCAM(analysis_return_intensity) {1}
set LWDAQ_info_BCAM(daq_source_device_type) {9}

#arbitrary initial values. List of LEDs to flash and minimum intensity threshold
set ledlist "all"
set min 500

proc go {} {
	#access variables used outside process
	global t min LWDAQ_config_BCAM ledlist

	#the following lines clear from past acquisitions variables which will be used later in the process
	#list of output intensities
	set outputlist ""
	#sum of [(intensity differences from the mean), squared]
	set varsum "0"
	#standard deviation
	set stdev "0"
	#running index for the sum used in varsum
	set varcount "0"
	#sum of all intensities, used in the calculation of avg and stdev
	set total "0"
	#running index for the number of LEDs flashed so far during an acquisition
	set resultcount "0"
	#list of LEDs which did not meet the minimum threshold during the acquisition
	set failedledlist ""

	#replaces 'all' with list of all LEDs
	if {$ledlist == "all"} {
		set ledlist "a1 a2 a3 a4 a5 a6 a7 a8 b1 b2 b3 b4 b5 b6 b7 b8 c1 c2 c3 c4 c5 c6 c7 c8 c9 c10 d1 d2 d3 d4 d5 d6 d7 d8 d9 d10"
	}

	#loops through each LED in the entered list of LEDs
 	foreach j $ledlist {

 		#set BCAM to LED indexed j in ledlist
		set LWDAQ_config_BCAM(daq_source_device_element) "$j"
		#acquire and take resulting intensity value
		set result [lindex [LWDAQ_acquire BCAM] 1]
		#print intensity
		LWDAQ_print $t "LED $j outputs $result" 

		#increments number of acquisitions taken
		set resultcount [expr {$resultcount + 1}]
		#adds result to total intensity 
		set total [expr {$total + $result}]
		#adds the output intensity value to a list of all outputs taken this acquisition
		lappend outputlist $result
		#alert message if LED output is below selected minimum threshold 
		if {$result < $min} {	
			LWDAQ_print $t "Output of $result from LED $j is below minimum of $min" red
			lappend failedledlist $j
		}
	}
	#runs if no LEDs were entered in LED list
	if {$resultcount == "0"} {
		LWDAQ_print $t "No LEDs listed - enter LED names separated by spaces in LED list" red
	#runs if data has been acquired
	} else {
		#average value
		set avg [expr {$total / $resultcount}]
		#standard deviation loop
		foreach n [lrange $outputlist 0 end] {
			#selects output intensity from list
			set output [lindex $outputlist $varcount]
			#adds square of difference of output with mean to running sum
			set varsum [expr {$varsum + (($output - $avg) ** 2)}]
			#increments sum index
			set varcount [expr {$varcount + 1}]
		}
		#divides total sum of sqared differences by sum index and takes sqare root yielding standard deviation
		set stdev [expr {round(sqrt($varsum / $varcount))}]
		LWDAQ_print $t "Average output is $avg and standard deviation of outputs for this acquisition is $stdev" blue
		if {[llength $failedledlist ] > 0} {
			LWDAQ_print $t "The following LEDs were below the minimum threshold of $min: $failedledlist" red
		}
	}
}




